@@ -7,18 +7,19 @@ use std::process;
 use std::sync::Arc;
 use vrp_cli::{get_errors_serialized, get_locations_serialized};
 use vrp_core::models::{Problem, Solution};
+use vrp_core::solver::Builder;
 use vrp_pragmatic::format::problem::{deserialize_problem, PragmaticProblem};
 use vrp_pragmatic::format::solution::PragmaticSolution;
 use vrp_scientific::common::read_init_solution;
 use vrp_scientific::lilim::{LilimProblem, LilimSolution};
 use vrp_scientific::solomon::{SolomonProblem, SolomonSolution};
-use vrp_solver::SolverBuilder;
 
 const FORMAT_ARG_NAME: &str = "FORMAT";
 const PROBLEM_ARG_NAME: &str = "PROBLEM";
 const MATRIX_ARG_NAME: &str = "matrix";
 const GENERATIONS_ARG_NAME: &str = "max-generations";
 const TIME_ARG_NAME: &str = "max-time";
+const COST_VARIATION_ARG_NAME: &str = "cost-variation";
 const GEO_JSON_ARG_NAME: &str = "geo-json";
 
 const INIT_SOLUTION_ARG_NAME: &str = "init-solution";
@@ -123,6 +124,14 @@ pub fn get_solve_app<'a, 'b>() -> App<'a, 'b> {
                 .required(false)
                 .takes_value(true),
         )
+        .arg(
+            Arg::with_name(COST_VARIATION_ARG_NAME)
+                .help("Specifies cost variation coefficient termination criteria in form \"sample_size,threshold\"")
+                .short("c")
+                .long(COST_VARIATION_ARG_NAME)
+                .required(false)
+                .takes_value(true),
+        )
         .arg(
             Arg::with_name(INIT_SOLUTION_ARG_NAME)
                 .help("Specifies path to file with initial solution")
@@ -177,16 +186,26 @@ pub fn run_solve(matches: &ArgMatches) {
     // optional
     let max_generations = matches.value_of(GENERATIONS_ARG_NAME).map(|arg| {
         arg.parse::<usize>().unwrap_or_else(|err| {
-            eprintln!("Cannot get max generations: '{}'", err.to_string());
+            eprintln!("cannot get max generations: '{}'", err.to_string());
             process::exit(1);
         })
     });
     let max_time = matches.value_of(TIME_ARG_NAME).map(|arg| {
         arg.parse::<usize>().unwrap_or_else(|err| {
-            eprintln!("Cannot get max time: '{}'", err.to_string());
+            eprintln!("cannot get max time: '{}'", err.to_string());
             process::exit(1);
         })
     });
+    let cost_variation = matches.value_of(COST_VARIATION_ARG_NAME).map(|arg| {
+        if let [sample, threshold] =
+            arg.split(',').filter_map(|line| line.parse::<f64>().ok()).collect::<Vec<_>>().as_slice()
+        {
+            (*sample as usize, *threshold)
+        } else {
+            eprintln!("cannot parse cost variation");
+            process::exit(1);
+        }
+    });
     let init_solution = matches.value_of(INIT_SOLUTION_ARG_NAME).map(|path| open_file(path, "init solution"));
     let matrix_files = matches
         .values_of(MATRIX_ARG_NAME)
@@ -202,7 +221,7 @@ pub fn run_solve(matches: &ArgMatches) {
 
             if is_get_locations_set {
                 locations_writer.0(problem_file, out_buffer).unwrap_or_else(|err| {
-                    eprintln!("Cannot get locations '{}'", err);
+                    eprintln!("cannot get locations '{}'", err);
                     process::exit(1);
                 });
             } else {
@@ -210,26 +229,30 @@ pub fn run_solve(matches: &ArgMatches) {
                     Ok(problem) => {
                         let problem = Arc::new(problem);
                         let solution = init_solution.and_then(|file| init_reader.0(file, problem.clone()));
-                        let solution = SolverBuilder::default()
-                            .with_init_solution(solution.map(|s| (problem.clone(), Arc::new(s))))
+                        let (solution, _) = Builder::default()
+                            .with_problem(problem.clone())
+                            .with_solutions(solution.map_or_else(|| vec![], |s| vec![Arc::new(s)]))
                             .with_max_generations(max_generations)
                             .with_max_time(max_time)
+                            .with_cost_variation(cost_variation)
                             .build()
-                            .solve(problem.clone());
-                        match solution {
-                            Some(solution) => solution_writer.0(&problem, solution.0, out_buffer, geo_buffer).unwrap(),
-                            None => println!("Cannot find any solution"),
-                        };
+                            .and_then(|solver| solver.solve())
+                            .unwrap_or_else(|err| {
+                                eprintln!("cannot find any solution: '{}'", err);
+                                process::exit(1);
+                            });
+
+                        solution_writer.0(&problem, solution, out_buffer, geo_buffer).unwrap()
                     }
                     Err(error) => {
-                        eprintln!("Cannot read {} problem from '{}': '{}'", problem_format, problem_path, error);
+                        eprintln!("cannot read {} problem from '{}': '{}'", problem_format, problem_path, error);
                         process::exit(1);
                     }
                 };
             }
         }
         None => {
-            eprintln!("Unknown format: '{}'", problem_format);
+            eprintln!("unknown format: '{}'", problem_format);
             process::exit(1);
         }
     }
